EN_BIT = 0x04 # 0000 0100 - Enable
RW_BIT = 0x02 # 0000 0010 - Read/Write
RS_BIT = 0x01 # 0000 0001 - Register selector

#  commands
LCD_CLEARDISPLAY    = 0x01 # 0000 0001
LCD_RETURNHOME      = 0x02 # 0000 0010
LCD_ENTRYMODESET    = 0x04 # 0000 0100
LCD_DISPLAYCONTROL  = 0x08 # 0000 1000
LCD_CURSORSHIFT     = 0x10 # 0001 0000
LCD_FUNCTIONSET     = 0x20 # 0010 0000
LCD_SET_CGRAM_ADDR  = 0x40 # 0100 0000 - Character Generator RAM
LCD_SET_DDRAM_ADDR  = 0x80 # 1000 0000 - Display Data RAM

# flags for display entry mode
LCD_ENTRYRIGHT          = 0x00 # 0000 0000
LCD_ENTRYLEFT           = 0x02 # 0000 0010
LCD_ENTRYSHIFTINCREMENT = 0x01 # 0000 0001
LCD_ENTRYSHIFTDECREMENT = 0x00 # 0000 0000

# flags for display on/off control
LCD_DISPLAYON       = 0x04 # 0000 0100
LCD_DISPLAYOFF      = 0x00 # 0000 0000
LCD_CURSORON        = 0x02 # 0000 0010
LCD_CURSOROFF       = 0x00 # 0000 0000
LCD_BLINKON         = 0x01 # 0000 0001
LCD_BLINKOFF        = 0x00 # 0000 0000

# flags for display/cursor shift
LCD_DISPLAYMOVE     = 0x08 # 0000 1000
LCD_CURSORMOVE      = 0x00 # 0000 0000
LCD_MOVERIGHT       = 0x04 # 0000 0100
LCD_MOVELEFT        = 0x00 # 0000 0000

# flags for function set
LCD_8BITMODE        = 0x10 # 0001 0000
LCD_4BITMODE        = 0x00 # 0000 0000
LCD_2LINE           = 0x08 # 0000 1000
LCD_1LINE           = 0x00 # 0000 0000
LCD_5x10DOTS        = 0x04 # 0000 0100
LCD_5x8DOTS         = 0x00 # 0000 0000

# flags for backlight control
LCD_BACKLIGHT       = 0x08 # 0000 1000
LCD_NOBACKLIGHT     = 0x00 # 0000 0000


from smbus2 import SMBus
from ..constants import EN_BIT
from .delay import delay_us


# /************ low level data pushing commands **********/
class LowLevel():
    def __init__(self, bus: SMBus, address: int, backlight: int) -> None:
        self._bus = bus
        self._address = address
        self._backlight = backlight
    
    # write either command or data
    def send(self, value: int, mode: int):
        high_nibble: int = value & 0xf0 #0xf0 is 11110000
        low_nibble: int = (value<<4) & 0xf0
        self.write4bits(high_nibble|mode)
        self.write4bits(low_nibble|mode)


    def write4bits(self, value: int):
        self.expanderWrite(value)
        self.pulseEnable(value)


    def expanderWrite(self, _data: int):                                        
        # Wire.beginTransmission(_Addr)
        # printIIC((int)(_data) | _backlightval)
        # Wire.endTransmission()
        self._bus.write_byte_data(self._address, 0, _data | self._backlight)


    def pulseEnable(self, _data: int):
        self.expanderWrite(_data | EN_BIT)  # ENABLE_BIT high
        delay_us(1) # enable pulse must be >450ns

        self.expanderWrite(_data & ~EN_BIT) # ENABLE_BIT low
        delay_us(50) # commands need > 37us to settle




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lcd_i2c', 'lcd_i2c.helpers']

package_data = \
{'': ['*']}

install_requires = \
['smbus2>=0.4.2,<0.5.0']

setup_kwargs = {
    'name': 'lcd-i2c',
    'version': '0.1.3',
    'description': 'Library for interacting with an I2C LCD screen through Python',
    'long_description': 'Basic library for interacting with I2C LCD screens.\n\nTested using a Raspberry Pi Zero 2W.\n',
    'author': 'Federico Giancarelli',
    'author_email': 'hello@federicogiancarelli.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
